/****************************************************************************
 * (c) 2018 softwareentwicklung-als-prozess.de
 * Funktionalitaet:
 * Siebensegment Display am 8574 Portexpander ansteuern
 ***************************************************************************/
// Includes -----------------------------------------------------------------
#include <Wire.h>
#include "sevseg.h"

// Defines -----------------------------------------------------------------
#define ALL_SEGS_ON 0xff
#define ALL_SEGS_OFF 0x00
#define PCF8574_ADDR (0x25)

// Variablen -----------------------------------------------------------------
unsigned char ledmask = ALL_SEGS_OFF;

//  internal Prototypes ------------------------------------------------------
static unsigned char invert_P3(unsigned char value);
static void SwitchSegs(unsigned char mask);

// Funktionen ----------------------------------------------------------------
void SEG_Setup(void)
{
  Wire.begin();  
}

void SEG_AllOn(void)
{
  ledmask = ALL_SEGS_ON;
  SwitchSegs(ledmask);
}

void SEG_AllOff(void)
{
  ledmask = ALL_SEGS_OFF;
  SwitchSegs(ledmask);
}

void SEG_Direkt(unsigned char value)
{
  ledmask = value;
  SwitchSegs(ledmask);
}

void SEG_On(unsigned char segno)
{
  switch(segno)
  {
    case SEG_A:
      ledmask |= SEG_A;
      break;
    case SEG_B:
      ledmask |= SEG_B;
      break;
    case SEG_C:
      ledmask |= SEG_C;
      break;
    case SEG_D:
      ledmask |= SEG_D;
      break;
    case SEG_E:
      ledmask |= SEG_E;
      break;
    case SEG_F:
      ledmask |= SEG_F;
      break;
    case SEG_G:
      ledmask |= SEG_G;
      break;
    case SEG_H:
      ledmask |= SEG_H;
      break;
    default:
      break;
  }
  SwitchSegs(ledmask);
}

void SEG_Off(unsigned char segno)
{
  switch(segno)
  {
    case SEG_A:
      ledmask &= ~SEG_A;
      break;
    case SEG_B:
      ledmask &= ~SEG_B;
      break;
    case SEG_C:
      ledmask &= ~SEG_C;
      break;
    case SEG_D:
      ledmask &= ~SEG_D;
      break;
    case SEG_E:
      ledmask &= ~SEG_E;
      break;
    case SEG_F:
      ledmask &= ~SEG_F;
      break;
    case SEG_G:
      ledmask &= ~SEG_G;
      break;
    case SEG_H:
      ledmask &= ~SEG_H;
      break;
    default:
      break;
  }
  SwitchSegs(ledmask);
}

// Port P3 wird ueber den Transistor im LCD Treibermodul invertiert
// Zusaetzlich wird ein Widerstand von 10 KOhm zwischen Pin 15 und Pin 16 
//             benoetigt um einen High Pegel erzeugen zu koennen
static unsigned char invert_P3(unsigned char value)
{
  unsigned char retval = value;
  
  if ((value & SEG_D) == (unsigned char)0)
  {
    retval |= SEG_D;
  }
  else
  {
    retval &= ~SEG_D;
  }
  return(retval);
}

static void SwitchSegs(unsigned char mask)
{
  mask = invert_P3(mask);
  Wire.beginTransmission(PCF8574_ADDR);
  Wire.write(mask);
  Wire.endTransmission();  
}

